@@ -10,8 +10,8 @@ from __future__ import with_statement
 import math, os, re, sys
 
 import colormath
-import x3dom
 from log import safe_print
+from options import debug, verbose
 from util_io import GzipFileProper, StringIOu as StringIO
 
 
@@ -66,8 +66,8 @@ def rcut(value, width):
 
 def sort_RGB_gray_to_top(a, b):
 	if a[0] == a[1] == a[2]:
-		if b[0] == b[1] == b[2] and a[0] > b[0]:
-			return 1
+		if b[0] == b[1] == b[2]:
+			return 0
 		return -1
 	else:
 		return 0
@@ -75,9 +75,9 @@ def sort_RGB_gray_to_top(a, b):
 
 def sort_RGB_to_top_factory(i1, i2, i3, i4):
 	def sort_RGB_to_top(a, b):
-		if a[i1] == a[i2] and not a[i3] and a[i4]:
-			if b[i1] == b[i2] and not b[i3] and b[i4] and a[i4] > b[i4]:
-				return 1
+		if a[i1] == a[i2] and 0 <= a[i3] < a[i4]:
+			if b[i1] == b[i2] and 0 <= b[i3] < b[i4]:
+				return 0
 			return -1
 		else:
 			return 0
@@ -95,8 +95,8 @@ def sort_RGB_white_to_top(a, b):
 def sort_by_HSI(a, b):
 	a = list(colormath.RGB2HSI(*a[:3]))
 	b = list(colormath.RGB2HSI(*b[:3]))
-	a[0] = round(a[0], 12)
-	b[0] = round(b[0], 12)
+	a[0] = round(math.degrees(a[0]))
+	b[0] = round(math.degrees(b[0]))
 	if a > b:
 		return 1
 	elif a < b:
@@ -108,8 +108,8 @@ def sort_by_HSI(a, b):
 def sort_by_HSL(a, b):
 	a = list(colormath.RGB2HSL(*a[:3]))
 	b = list(colormath.RGB2HSL(*b[:3]))
-	a[0] = round(a[0], 12)
-	b[0] = round(b[0], 12)
+	a[0] = round(math.degrees(a[0]))
+	b[0] = round(math.degrees(b[0]))
 	if a > b:
 		return 1
 	elif a < b:
@@ -121,8 +121,8 @@ def sort_by_HSL(a, b):
 def sort_by_HSV(a, b):
 	a = list(colormath.RGB2HSV(*a[:3]))
 	b = list(colormath.RGB2HSV(*b[:3]))
-	a[0] = round(a[0], 12)
-	b[0] = round(b[0], 12)
+	a[0] = round(math.degrees(a[0]))
+	b[0] = round(math.degrees(b[0]))
 	if a > b:
 		return 1
 	elif a < b:
@@ -140,6 +140,15 @@ def sort_by_RGB(a, b):
 		return 0
 
 
+def sort_by_BGR(a, b):
+	if a[:3][::-1] > b[:3][::-1]:
+		return 1
+	elif a[:3] == b[:3]:
+		return 0
+	else:
+		return -1
+
+
 def sort_by_RGB_sum(a, b):
 	sum1, sum2 = sum(a[:3]), sum(b[:3])
 	if sum1 > sum2:
@@ -150,6 +159,16 @@ def sort_by_RGB_sum(a, b):
 		return 0
 
 
+def sort_by_RGB_pow_sum(a, b):
+	sum1, sum2 = sum(v ** 2.2 for v in a[:3]), sum(v ** 2.2 for v in b[:3])
+	if sum1 > sum2:
+		return 1
+	elif sum1 < sum2:
+		return -1
+	else:
+		return 0
+
+
 def sort_by_L(a, b):
 	Lab1 = colormath.XYZ2Lab(*a[3:])
 	Lab2 = colormath.XYZ2Lab(*b[3:])
@@ -161,6 +180,22 @@ def sort_by_L(a, b):
 		return 0
 
 
+def sort_by_luma_factory(RY, GY, BY, gamma=1):
+	def sort_by_luma(a, b):
+		a = RY * a[0] ** gamma + GY * a[1] ** gamma + BY * a[2] ** gamma
+		b = RY * b[0] ** gamma + GY * b[1] ** gamma + BY * b[2] ** gamma
+		if a > b:
+			return 1
+		elif a < b:
+			return -1
+		else:
+			return 0
+	return sort_by_luma
+
+
+sort_by_rec709_luma = sort_by_luma_factory(0.2126, 0.7152, 0.0722)
+
+
 class CGATSError(Exception):
 	pass
 
@@ -207,7 +242,8 @@ class CGATS(dict):
 	type = 'ROOT'
 	vmaxlen = 0
 	
-	def __init__(self, cgats=None, normalize_fields=False, file_identifier="CTI3"):
+	def __init__(self, cgats=None, normalize_fields=False, file_identifier="CTI3",
+				 emit_keywords=False):
 		"""
 		Return a CGATS instance.
 		
@@ -223,6 +259,7 @@ class CGATS(dict):
 		
 		self.normalize_fields = normalize_fields
 		self.file_identifier = file_identifier
+		self.emit_keywords = emit_keywords
 		self.root = self
 		self._keys = []
 		
@@ -252,6 +289,8 @@ class CGATS(dict):
 			context = self
 
 			for raw_line in raw_lines:
+				# Replace 1.#IND00 with NaN
+				raw_line = raw_line.replace("1.#IND00", "NaN")
 				# strip control chars and leading/trailing whitespace
 				line = re.sub('[^\x09\x20-\x7E\x80-\xFF]', '', 
 								raw_line.strip())
@@ -407,7 +446,7 @@ class CGATS(dict):
 			self.setmodified(value)
 		elif name in ('datetime', 'filename', 'fileName', 'file_identifier', 'key', 
 					  'mtime', 'normalize_fields', 'parent', 'root', 'type', 
-					  'vmaxlen'):
+					  'vmaxlen', 'emit_keywords'):
 			object.__setattr__(self, name, value)
 			self.setmodified()
 		else:
@@ -463,12 +502,9 @@ class CGATS(dict):
 						else:
 							if 'KEYWORDS' in self and \
 								key in self['KEYWORDS'].values():
-								result.append('KEYWORD "%s"' % key)
-								result.append('%s "%s"' % (key, value))
-							elif type(value) in (int, float):
-								result.append('%s %s' % (key, value))
-							else:
-								result.append('%s "%s"' % (key, value))
+								if self.emit_keywords:
+									result.append('KEYWORD "%s"' % key)
+							result.append('%s "%s"' % (key, value))
 				elif key not in ('DATA_FORMAT', 'KEYWORDS'):
 					if (value.type == 'SECTION' and result[-1:] and 
 						result[-1:][0] != ''):
@@ -479,7 +515,7 @@ class CGATS(dict):
 			if self.type == 'SECTION' or data:
 				result.append('')
 		if data and data.parent['DATA_FORMAT']:
-			if 'KEYWORDS' in data.parent:
+			if 'KEYWORDS' in data.parent and self.emit_keywords:
 				for item in data.parent['DATA_FORMAT'].values():
 					if item in data.parent['KEYWORDS'].values():
 						result.append('KEYWORD "%s"' % item)
@@ -563,7 +599,7 @@ class CGATS(dict):
 		field_names = ("RGB_R", "RGB_G", "RGB_B", "XYZ_X", "XYZ_Y", "XYZ_Z")
 		data = self.get_data(field_names)
 		if not data:
-			return False
+			return False, False
 		valueslist = []
 		for key, item in data.iteritems():
 			values = []
@@ -577,33 +613,158 @@ class CGATS(dict):
 		for i, values in enumerate(valueslist):
 			for j, field_name in enumerate(field_names):
 				self[i][field_name] = values[j]
+		return True
 	
-	def checkerboard(self):
+	def checkerboard(self, sort1=sort_by_L, sort2=sort_RGB_white_to_top,
+					 split_grays=False, shift=False):
 		data, valueslist = self.get_RGB_XYZ_values()
 		if not valueslist:
 			return False
-		valueslist *= 2
-		valueslist.sort(sort_by_L)
-		valueslist.sort(sort_RGB_white_to_top)
-		split = len(valueslist) / 2
-		valueslist1 = valueslist[:split]
-		valueslist2 = valueslist[split:]
-		valueslist2.reverse()
-		valueslist = valueslist1 + valueslist2
+		numvalues = len(valueslist)
+		if sort1:
+			valueslist.sort(sort1)
+		if sort2:
+			valueslist.sort(sort2)
+		gray = []
+		if split_grays:
+			# Split values into gray and color. First gray in a consecutive
+			# sequence of two or more grays will be added to color list,
+			# following grays will be added to gray list.
+			color = []
+			prev_i = -1
+			prev_values = []
+			added = {prev_i: True}  # Keep track of entries we have added
+			for i, values in enumerate(valueslist):
+				if debug:
+					safe_print(i + 1, "IN", values[:3])
+				is_gray = values[:3] == [values[:3][0]] * 3
+				prev = color
+				cur = color
+				if is_gray:
+					if not prev_values:
+						if debug:
+							safe_print("WARNING - skipping gray because no prev")
+					elif values[:3] == prev_values[:3]:
+						# Same gray as prev value
+						prev = color
+						cur = gray
+						if prev_i not in added:
+							if debug:
+								safe_print("INFO - appending prev %s to color "
+										   "because prev was same gray but got "
+										   "skipped" % prev_values[:3])
+						if debug:
+							safe_print("INFO - appending cur to gray because "
+									   "prev %s was same gray" %
+									   prev_values[:3])
+					elif prev_values[:3] == [prev_values[:3][0]] * 3:
+						# Prev value was different gray
+						prev = gray
+						cur = gray
+						if prev_i not in added:
+							if debug:
+								safe_print("INFO - appending prev %s to gray "
+										   "because prev was different gray "
+										   "but got skipped" % prev_values[:3])
+						if debug:
+							safe_print("INFO - appending cur to gray because "
+									   "prev %s was different gray" %
+									   prev_values[:3])
+					elif i < numvalues - 1:
+						if debug:
+							safe_print("WARNING - skipping gray because "
+									   "prev %s was not gray" %
+									   prev_values[:3])
+					else:
+						# Last
+						if debug:
+							safe_print("INFO - appending cur to color "
+									   "because prev %s was not gray but "
+									   "cur is last" % prev_values[:3])
+				if not is_gray or cur is gray or i == numvalues - 1:
+					if prev_i not in added:
+						if debug and prev is cur is color:
+							safe_print("INFO - appending prev %s to color because "
+									   "prev got skipped" % prev_values[:3])
+						prev.append(prev_values)
+						added[prev_i] = True
+					if debug and not is_gray and cur is color:
+						safe_print("INFO - appending cur to color")
+					cur.append(values)
+					added[i] = True
+				prev_i = i
+				prev_values = values
+			if (len(color) == 2 and color[0][:3] == [0, 0, 0] and
+				color[1][:3] == [100, 100, 100]):
+				if debug:
+					safe_print("INFO - appending color to gray because color "
+							   "is only black and white")
+				gray.extend(color)
+				color = []
+				if sort1:
+					gray.sort(sort1)
+				if sort2:
+					gray.sort(sort2)
+			if debug:
+				for i, values in enumerate(gray):
+					safe_print("%4i" % (i + 1), "GRAY", ("%8.4f " * 3) %
+							   tuple(values[:3]))
+				for i, values in enumerate(color):
+					safe_print("%4i" % (i + 1), "COLOR", ("%8.4f " * 3) %
+							   tuple(values[:3]))
+		else:
+			color = valueslist
 		checkerboard = []
-		for i in xrange(split):
-			if i % 2 == 1:
-				i = len(valueslist) - 1 - i
-			values = valueslist[i]
-			checkerboard.append(values)
+		for valueslist in [gray, color]:
+			if not valueslist:
+				continue
+			split = int(round(len(valueslist) / 2.0))
+			valueslist1 = valueslist[:split]
+			valueslist2 = valueslist[split:]
+			if shift:
+				# Shift values.
+				#
+				# If split is even:
+				#   A1 A2 A3 A4 -> A1 B2 B3 B1 B4
+				#   B1 B2 B3 B4 -> A3 A4 A2
+				#
+				# If split is uneven:
+				#   A1 A2 A3 -> A1 B1 B2 B3 B4
+				#   B1 B2 B3 B4 -> A2 A3
+				offset = 0
+				if split == len(valueslist) / 2.0:
+					# Even split
+					offset += 1
+				valueslist1_orig = list(valueslist1)
+				valueslist2_orig = list(valueslist2)
+				valueslist1 = valueslist2_orig[offset:]
+				valueslist2 = valueslist1_orig[offset + 1:]
+				valueslist1.insert(0, valueslist1_orig[0])
+				if offset:
+					valueslist1.insert(-1, valueslist2_orig[0])
+					valueslist2.extend(valueslist1_orig[1:2])
+			# Interleave.
+			# 1 2 3 4 5 6 7 8 -> 1 5 2 6 3 7 4 8
+			while valueslist1 or valueslist2:
+				for valueslist in (valueslist1, valueslist2):
+					if valueslist:
+						values = valueslist.pop(0)
+						checkerboard.append(values)
+		if (shift and
+			checkerboard[-1][:3] == [100, 100, 100]):
+			# Move white patch to front
+			if debug:
+				safe_print("INFO - moving white to front")
+			checkerboard.insert(0, checkerboard.pop())
+		if len(checkerboard) != numvalues:
+			# This should never happen
+			safe_print("Number of patches incorrect after re-ordering (is %i, "
+					   "should be %i)" % (len(checkerboard), numvalues))
+			return False
 		return data.set_RGB_XYZ_values(checkerboard)
 	
 	def sort_RGB_gray_to_top(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_RGB_gray_to_top)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_RGB_gray_to_top)
 	
 	def sort_RGB_to_top(self, r=0, g=0, b=0):
 		"""
@@ -613,9 +774,6 @@ class CGATS(dict):
 		Example: sort_RGB_to_top(0, True, True) - sort cyan values to top
 		
 		"""
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
 		if r and g and b:
 			fn = sort_RGB_gray_to_top
 		elif r and g:
@@ -630,56 +788,44 @@ class CGATS(dict):
 			fn = sort_RGB_to_top_factory(0, 2, 0, 1)
 		elif b:
 			fn = sort_RGB_to_top_factory(0, 1, 0, 2)
-		valueslist.sort(fn)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(fn)
 	
 	def sort_RGB_white_to_top(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_RGB_white_to_top)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_RGB_white_to_top)
 	
 	def sort_by_HSI(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_by_HSI)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_by_HSI)
 	
 	def sort_by_HSL(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_by_HSL)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_by_HSL)
 	
 	def sort_by_HSV(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_by_HSV)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_by_HSV)
 	
 	def sort_by_L(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_by_L)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_by_L)
 	
 	def sort_by_RGB(self):
-		data, valueslist = self.get_RGB_XYZ_values()
-		if not valueslist:
-			return False
-		valueslist.sort(sort_by_RGB)
-		return data.set_RGB_XYZ_values(valueslist)
+		return self.sort_data_RGB_XYZ(sort_by_RGB)
+	
+	def sort_by_BGR(self):
+		return self.sort_data_RGB_XYZ(sort_by_BGR)
+	
+	def sort_by_RGB_pow_sum(self):
+		return self.sort_data_RGB_XYZ(sort_by_RGB_pow_sum)
 	
 	def sort_by_RGB_sum(self):
+		return self.sort_data_RGB_XYZ(sort_by_RGB_sum)
+	
+	def sort_by_rec709_luma(self):
+		return self.sort_data_RGB_XYZ(sort_by_rec709_luma)
+
+	def sort_data_RGB_XYZ(self, cmp=None, key=None, reverse=False):
+		""" Sort RGB/XYZ data """
 		data, valueslist = self.get_RGB_XYZ_values()
 		if not valueslist:
 			return False
-		valueslist.sort(sort_by_RGB_sum)
+		valueslist.sort(cmp, key, reverse)
 		return data.set_RGB_XYZ_values(valueslist)
 	
 	@property
@@ -755,7 +901,17 @@ class CGATS(dict):
 													  'got %s)' % 
 													  (item, type(value)))
 							else:
-								lencheck = len(str(abs(value)).split("e")[0])
+								strval = str(abs(value))
+								if (item.startswith("RGB_") or
+									item.startswith("CMYK_")):
+									# Assuming 0..100, 4 decimal digits is
+									# enough for roughly 19 bits integer
+									# device values
+									parts = strval.split(".")
+									if len(parts) == 2 and len(parts[-1]) > 4:
+										value = round(value, 4)
+										strval = str(abs(value))
+								lencheck = len(strval.split("e")[0])
 								if lencheck > self.vmaxlen:
 									self.vmaxlen = lencheck
 						elif self.root.normalize_fields and \
@@ -836,13 +992,24 @@ class CGATS(dict):
 							elif value == 'SAMPLENAME':
 								value = 'SAMPLE_NAME'
 						if var == 'KEYWORD':
+							self.emit_keywords = True
 							if value != 'KEYWORD':
 								self.add_keyword(value)
 							else:
 								safe_print('Warning: cannot add keyword '
 											'"KEYWORD"')
 						else:
-							if isinstance(value, basestring):
+							if (isinstance(value, basestring) and
+								key not in ("DESCRIPTOR", "ORIGINATOR",
+										    "CREATED", "DEVICE_CLASS",
+										    "COLOR_REP", "TARGET_INSTRUMENT",
+										    "LUMINANCE_XYZ_CDM2", "OBSERVER",
+											"INSTRUMENT", "MANUFACTURER_ID",
+											"MANUFACTURER", "REFERENCE",
+											"REFERENCE_OBSERVER", "DISPLAY",
+											"TECHNOLOGY", "REFERENCE_FILENAME",
+											"REFERENCE_HASH", "TARGET_FILENAME",
+											"TARGET_HASH", "FIT_METHOD")):
 								match = re.match(
 									'(?:\d+|((?:\d*\.\d+|\d+)(?:e[+-]?\d+)?))$', value)
 								if match:
@@ -851,7 +1018,7 @@ class CGATS(dict):
 									else:
 										value = int(value)
 									if self.type in ('DATA_FORMAT', 
-														'KEYWORDS'):
+													'KEYWORDS'):
 										raise CGATSTypeError('Invalid data '
 															 'type for %s '
 															 '(expected str '
@@ -873,8 +1040,9 @@ class CGATS(dict):
 				  normalize_RGB_white=False, compress=True, format="VRML"):
 		if colorspace not in ("DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
 							  "LCH(uv)", "Lab", "Luv", "Lu'v'", "RGB", "xyY",
-							  "HSI", "HSL", "HSV", "ICtCp"):
+							  "HSI", "HSL", "HSV", "ICtCp", "IPT", "Lpt"):
 			raise ValueError("export_3d: Unknown colorspace %r" % colorspace)
+		import x3dom
 		data = self.queryv1("DATA")
 		if self.queryv1("ACCURATE_EXPECTED_VALUES") == "true":
 			cat = "Bradford"
@@ -935,7 +1103,7 @@ Transform {
 		}
 """
 		axes = ""
-		if (colorspace not in ("Lab", "Luv", "ICtCp") and
+		if (colorspace not in ("Lab", "Luv", "ICtCp", "IPT", "Lpt") and
 			not colorspace.startswith("DIN99")):
 			if colorspace in ("Lu'v'", "xyY"):
 				maxz = scale = 100
@@ -1005,10 +1173,23 @@ Transform {
 				nxcolor = "0.8 1.0 0.0"
 				pycolor = "1.0 0.0 0.25"
 				nycolor = "0.0 1.0 1.0"
+			elif colorspace == "IPT":
+				(pxlabel,
+				 nxlabel,
+				 pylabel,
+				 nylabel,
+				 pllabel) = ('"P", "+%.1f"' % 1,
+							 '"P", "-%.1f"' % 1,
+							 '"T +%.1f"' % 1,
+							 '"T -%.1f"' % 1,
+							 '"I"')
 			else:
 				if colorspace == "Luv":
 					x = "u"
 					y = "v"
+				elif colorspace == "Lpt":
+					x = "p"
+					y = "t"
 				else:
 					x = "a"
 					y = "b"
@@ -1200,6 +1381,8 @@ Transform {
 									  entry["XYZ_Y"],
 									  entry["XYZ_Z"],
 									  white,
+									  "D65" if colorspace in ("ICtCp", "IPT")
+									  else "D50",
 									  cat=cat)
 			L, a, b = colormath.XYZ2Lab(X, Y, Z)
 			if colorspace == "RGB":
@@ -1272,6 +1455,12 @@ Transform {
 				I, Ct, Cp = colormath.XYZ2ICtCp(X / 100.0, Y / 100.0, Z / 100.0,
 												clamp=False)
 				x, y, z = Ct * 100, Cp * 100, I * 100 - 50
+			elif colorspace == "IPT":
+				I, P, T = colormath.XYZ2IPT(X / 100.0, Y / 100.0, Z / 100.0)
+				x, y, z = P * 100, T * 100, I * 100 - 50
+			elif colorspace == "Lpt":
+				L, p, t = colormath.XYZ2Lpt(X, Y, Z)
+				x, y, z = p, t, L - 50
 			if RGB_black_offset != 40:
 				# Keep reference hue and saturation
 				# Lab to sRGB using reference black offset of 40 like Argyll CMS
@@ -1456,6 +1645,80 @@ Transform {
 		dict.pop(self, name)
 		self.setmodified()
 		return result
+
+	def fix_zero_measurements(self, warn_only=False, logfile=safe_print):
+		"""
+		Fix (or warn about) zero measurements
+		
+		If XYZ/Lab = 0, the sample gets removed. If only one component of
+		XYZ/Lab is 0, it gets fudged so that the component is nonzero
+		(because otherwise, Argyll's colprof will remove it, which can have bad
+		effects if it's an 'essential' sample)
+		
+		"""
+		color_rep = (self.queryv1("COLOR_REP") or "").split("_")
+		data = self.queryv1("DATA")
+		if len(color_rep) == 2 and data:
+			# Check for XYZ/Lab = 0 readings
+			cie_labels = []
+			for channel in color_rep[1]:
+				cie_labels.append(color_rep[1] + "_" + channel)
+			device_labels = []
+			for channel in color_rep[0]:
+				device_labels.append(color_rep[0] + "_" + channel)
+			errors = []
+			remove = []
+			for key, sample in data.iteritems():
+				cie_values = [sample[label] for label in cie_labels]
+				# Check if zero
+				if filter(lambda v: v, cie_values):
+					# Not zero. Check if one component zero
+					if 0.0 in cie_values:
+						for label in cie_labels:
+							if not sample[label]:
+								if warn_only:
+									if logfile:
+										logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+													  (sample.SAMPLE_ID,
+													   " ".join(str(sample.queryv1(device_labels)).split()),
+													   label))
+								else:
+									# Fudge to be nonzero
+									sample[label] = 0.000001
+									if logfile:
+										logfile.write("Fudged sample ID %i (RGB %s) %s to be non-zero\n" %
+													  (sample.SAMPLE_ID,
+													   " ".join(str(sample.queryv1(device_labels)).split()),
+													   label))
+					continue
+				# Zero
+				device_values = [sample[label] for label in device_labels]
+				device_sum = 0
+				for value in device_values:
+					if value > 0:
+						# Error on device values > 0
+						errors.append(sample)
+					device_sum += value
+				if not device_sum:
+					# Skip device black
+					continue
+				if warn_only:
+					if logfile:
+						logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+									  (sample.SAMPLE_ID,
+									   " ".join(str(sample.queryv1(device_labels)).split()),
+									   color_rep[1]))
+				else:
+					# Queue sample for removal
+					remove.insert(0, sample)
+					if logfile:
+						logfile.write("Removed sample ID %i (RGB %s) with %s = 0\n" %
+									  (sample.SAMPLE_ID,
+									   " ".join(str(sample.queryv1(device_labels)).split()),
+									   color_rep[1]))
+			for sample in remove:
+				# Remove sample
+				data.pop(sample)
 	
 	def fix_device_values_scaling(self, color_rep=None):
 		"""
@@ -1598,8 +1861,14 @@ Transform {
 					values = colormath.Lab2XYZ(*values)
 				else:
 					values = [v / max_v for v in values]
-				values = colormath.apply_bpc(values[0], values[1], values[2], black,
-											bp_out, white, weight)
+				if weight:
+					values = colormath.apply_bpc(values[0], values[1],
+												 values[2], black, bp_out,
+												 white, weight)
+				else:
+					values = colormath.blend_blackpoint(values[0], values[1],
+														values[2], black,
+														bp_out)
 				values = [v * max_v for v in values]
 				if is_Lab:
 					values = colormath.XYZ2Lab(*values)
