@@ -155,10 +155,9 @@ resfiles = [
 	"ref/sRGB.gam",
 	"ref/verify_extended.ti1",
 	"ti1/d3-e4-s2-g25-m0-b0-f0.ti1",
-	"ti1/d3-e4-s0-g25-m3-b3-f0-crossover.ti1",
-	"ti1/d3-e4-s0-g49-m3-b3-f0-crossover.ti1",
-	"ti1/d3-e4-s13-g37-m4-b4-f0.ti1",
-	"ti1/d3-e4-s17-g49-m5-b5-f0.ti1",
+	"ti1/d3-e4-s3-g49-m3-b0-f0.ti1",
+	"ti1/d3-e4-s4-g49-m4-b0-f0.ti1",
+	"ti1/d3-e4-s5-g49-m5-b0-f0.ti1",
 	"xrc/extra.xrc",
 	"xrc/gamap.xrc",
 	"xrc/main.xrc",
@@ -188,32 +187,33 @@ untethered_displays = non_argyll_displays + ("Web$", "Chromecast ", "Prisma ",
 virtual_displays = untethered_displays + ("madVR$", )
 
 
-def is_special_display(display_no=None, tests=virtual_displays):
-	display_name = get_display_name(display_no)
+def is_special_display(display=None, tests=virtual_displays):
+	if not isinstance(display, basestring):
+		display = get_display_name(display)
 	for test in tests:
-		if re.match(test, display_name):
+		if re.match(test, display):
 			return True
 	return False
 
 
-def is_uncalibratable_display(display_no=None):
-	return is_special_display(display_no, uncalibratable_displays)
+def is_uncalibratable_display(display=None):
+	return is_special_display(display, uncalibratable_displays)
 
 
-def is_patterngenerator(display_no=None):
-	return is_special_display(display_no, patterngenerators)
+def is_patterngenerator(display=None):
+	return is_special_display(display, patterngenerators)
 
 
-def is_non_argyll_display(display_no=None):
-	return is_special_display(display_no, non_argyll_displays)
+def is_non_argyll_display(display=None):
+	return is_special_display(display, non_argyll_displays)
 
 
-def is_untethered_display(display_no=None):
-	return is_special_display(display_no, untethered_displays)
+def is_untethered_display(display=None):
+	return is_special_display(display, untethered_displays)
 
 
-def is_virtual_display(display_no=None):
-	return is_special_display(display_no, virtual_displays)
+def is_virtual_display(display=None):
+	return is_special_display(display, virtual_displays)
 
 
 def check_3dlut_format(devicename):
@@ -575,7 +575,7 @@ def runtimeconfig(pyfile):
 	
 	"""
 	from log import setup_logging
-	setup_logging(logdir, pyname, pyext)
+	setup_logging(logdir, pyname, pyext, confighome=confighome)
 	if debug or verbose >= 1:
 		from log import safe_print
 	if debug:
@@ -650,8 +650,10 @@ valid_ranges = {
 	"measurement_report.trc_output_offset": [0.0, 1.0],
 	"measure.display_settle_time_mult": [0.000001, 10000.0],
 	"measure.min_display_update_delay_ms": [20, 60000],
+	"multiprocessing.max_cpus": [0, 65],
 	"patterngenerator.apl": [0.0, 1.0],
 	"patterngenerator.resolve.port": [1, 65535],
+	"profile_loader.quantize_bits": [8, 16],
 	"synthprofile.trc_gamma": [0.01, 10],
 	"synthprofile.trc_output_offset": [0.0, 1.0],
 	"tc_export_repeat_patch_max": [1, 1000],
@@ -675,6 +677,7 @@ valid_values = {
 									video_encodings),
 	"3dlut.format": ["3dl", "cube", "eeColor", "icc", "madVR", "mga", "png",
 					 "ReShade", "spi3d"],
+	"3dlut.hdr_display": [0, 1],
 	"3dlut.image.layout": ["h", "v"],
 	"3dlut.image.order": ["rgb", "bgr"],
 	"3dlut.rendering_intent": intents,
@@ -701,10 +704,11 @@ valid_values = {
 	"gamap_out_viewcond": ["mt", "mb", "md", "jm", "jd"],
 	"measurement_report.trc_gamma_type": ["b", "B"],
 	"observer": observers,
+	"patterngenerator.detect_video_levels": [0, 1],
 	"patterngenerator.prisma.preset": ["Movie", "Sports", "Game", "Animation",
 									   "PC/Mac", "Black+White", "Custom-1",
 									   "Custom-2"],
-	"patterngenerator.resolve.use_video_levels": [0, 1],
+	"patterngenerator.use_video_levels": [0, 1],
 	"profile.black_point_compensation": [0, 1],
 	"profile.install_scope": ["l", "u"],
 	"profile.quality": ["l", "m", "h", "u"],
@@ -716,9 +720,15 @@ valid_values = {
 	"tc_algo": ["", "t", "r", "R", "q", "Q", "i", "I"],  # Q = Argyll >= 1.1.0
 	"tc_vrml_use_D50": [0, 1],
 	"tc_vrml_cie_colorspace": ["DIN99", "DIN99b", "DIN99c", "DIN99d", "ICtCp",
-							   "LCH(ab)", "LCH(uv)", "Lab", "Luv", "Lu'v'", "xyY"],
+							   "IPT", "LCH(ab)", "LCH(uv)", "Lab", "Lpt", "Luv",
+							   "Lu'v'", "xyY"],
 	"tc_vrml_device_colorspace": ["HSI", "HSL", "HSV", "RGB"],
-	"testchart.auto_optimize": [0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18],
+	"testchart.auto_optimize": range(19),
+	"testchart.patch_sequence": ["optimize_display_response_delay",
+								 "maximize_lightness_difference",
+								 "maximize_rec709_luma_difference",
+								 "maximize_RGB_difference",
+								 "vary_RGB_difference"],
 	"trc": ["240", "709", "l", "s", ""],
 	"trc.type": ["g", "G"],
 	"uniformity.cols": [3, 5, 7, 9],
@@ -757,6 +767,7 @@ defaults = {
 	"3dlut.encoding.output.backup": "n",
 	"3dlut.format": "cube",
 	"3dlut.gamap.use_b2a": 0,
+	"3dlut.hdr_display": 0,
 	"3dlut.hdr_maxcll": 10000,
 	"3dlut.hdr_peak_luminance": 400,
 	"3dlut.image.layout": "h",
@@ -771,6 +782,8 @@ defaults = {
 	"3dlut.size": 65,
 	"3dlut.tab.enable": 0,
 	"3dlut.tab.enable.backup": 0,
+	"3dlut.whitepoint.x": 0.3127,
+	"3dlut.whitepoint.y": 0.329,
 	"allow_skip_sensor_cal": 0,
 	"app.allow_network_clients": 0,
 	"app.dpi": get_default_dpi(),
@@ -804,6 +817,7 @@ defaults = {
 	"calibration.update": 0,
 	"calibration.use_video_lut": 1,
 	"calibration.use_video_lut.backup": 1,
+	"ccmx.use_four_color_matrix_method": 0,
 	"colorimeter_correction.instrument": None,
 	"colorimeter_correction.instrument.reference": None,
 	"colorimeter_correction.measurement_mode": "l",
@@ -819,8 +833,8 @@ defaults = {
 	"comport.number": 1,
 	"comport.number.backup": 1,
 	# Note: worker.Worker.enumerate_displays_and_ports() overwrites copyright
-	"copyright": "No copyright. Created with %s %s and Argyll CMS" % (appname, 
-																	  version),
+	"copyright": "No copyright. Created with %s %s and ArgyllCMS" % (appname, 
+																	 version),
 	"dimensions.measureframe": "0.5,0.5,1.5",
 	"dimensions.measureframe.unzoomed": "0.5,0.5,1.5",
 	"dimensions.measureframe.whitepoint.visual_editor": "0.5,0.5,1.5",
@@ -890,17 +904,18 @@ defaults = {
 	"measure.override_display_settle_time_mult.backup": 0,
 	"measure.override_min_display_update_delay_ms": 0,
 	"measure.override_min_display_update_delay_ms.backup": 0,
+	"multiprocessing.max_cpus": 0,
 	"observer": "1931_2",
 	"observer.backup": "1931_2",
 	"patterngenerator.apl": .22,
+	"patterngenerator.detect_video_levels": 1,
 	"patterngenerator.prisma.argyll": 0,
 	"patterngenerator.prisma.host": "",
 	"patterngenerator.prisma.preset": "Custom-1",
 	"patterngenerator.prisma.port": 80,
-	"patterngenerator.prisma.use_video_levels": 1,
 	"patterngenerator.resolve": "CM",
 	"patterngenerator.resolve.port": 20002,
-	"patterngenerator.resolve.use_video_levels": 0,
+	"patterngenerator.use_video_levels": 0,
 	"position.x": 50,
 	"position.y": 50,
 	"position.info.x": 50,
@@ -922,6 +937,7 @@ defaults = {
 	"position.tcgen.x": 50,
 	"position.tcgen.y": 50,
 	"profile.black_point_compensation": 0,
+	"profile.black_point_correction": 0.0,
 	"profile.create_gamut_views": 1,
 	"profile.install_scope": "l" if (sys.platform != "win32" and 
 									 os.geteuid() == 0) # or 
@@ -952,11 +968,11 @@ defaults = {
 	"profile.save_path": storage, # directory
 	"profile.type": "X",
 	"profile.update": 0,
-	"profile_loader.buggy_video_drivers":  ";".join(["intel"]),
-	"profile_loader.check_gamma_ramps": 0,
+	"profile_loader.buggy_video_drivers":  ";".join(["*"]),
+	"profile_loader.check_gamma_ramps": 1,
 	"profile_loader.error.show_msg": 1,
 	"profile_loader.exceptions": "",
-	"profile_loader.fix_profile_associations": 0,
+	"profile_loader.fix_profile_associations": 1,
 	"profile_loader.known_apps": ";".join(["basiccolor display.exe",
 										   "calclient.exe",
 										   "coloreyes display pro.exe",
@@ -981,9 +997,13 @@ defaults = {
 										   "spyder5pro.exe",
 										   "dispcal.exe",
 										   "dispread.exe",
-										   "flux.exe"]),
+										   "dispwin.exe",
+										   "flux.exe",
+										   "dccw.exe"]),
 	"profile_loader.known_window_classes": ";".join(["CalClient.exe"]),
+	"profile_loader.quantize_bits": 16,
 	"profile_loader.reset_gamma_ramps": 0,
+	"profile_loader.track_other_processes": 1,
 	"profile_loader.use_madhcnet": 0,
 	"profile_loader.verify_calibration": 0,
 	"recent_cals": "",
@@ -1052,9 +1072,10 @@ defaults = {
 	"tc_vrml_use_D50": 0,
 	"tc_white_patches": 4,
 	"tc.show": 0,
-	"testchart.auto_optimize": 5,
+	"testchart.auto_optimize": 4,
 	"testchart.file": "auto",
 	"testchart.file.backup": "auto",
+	"testchart.patch_sequence": "optimize_display_response_delay",
 	"testchart.reference": "",
 	"ti3.check_sanity.auto": 0,
 	"trc": 2.2,
@@ -1075,17 +1096,17 @@ defaults = {
 	"use_separate_lut_access": 0,
 	"vrml.compress": 1,
 	"webserver.portnumber": 8080,
-	"whitepoint.colortemp": 5000,
-	"whitepoint.colortemp.backup": 5000,
+	"whitepoint.colortemp": 6500,
+	"whitepoint.colortemp.backup": 6500,
 	"whitepoint.colortemp.locus": "t",
 	"whitepoint.visual_editor.bg_v": 255,
 	"whitepoint.visual_editor.b": 255,
 	"whitepoint.visual_editor.g": 255,
 	"whitepoint.visual_editor.r": 255,
-	"whitepoint.x": 0.345741,
-	"whitepoint.x.backup": 0.345741,
-	"whitepoint.y": 0.358666,
-	"whitepoint.y.backup": 0.358666,
+	"whitepoint.x": 0.3127,
+	"whitepoint.x.backup": 0.3127,
+	"whitepoint.y": 0.3290,
+	"whitepoint.y.backup": 0.3290,
 	"x3dom.cache": 1,
 	"x3dom.embed": 0
 }
@@ -1313,7 +1334,7 @@ def get_standard_profiles(paths_only=False):
 						filename.startswith("gracol") or
 						filename.startswith("iso") or
 						filename.startswith("lstar-") or
-						filename.startswith("pso_") or
+						filename.startswith("pso") or
 						filename.startswith("prophoto") or
 						filename.startswith("psr_") or
 						filename.startswith("psrgravure") or
