@@ -33,13 +33,13 @@ from config import (autostart_home, confighome, datahome, enc, exe, exe_ext,
 from debughelpers import ResourceError, handle_error
 from log import log, safe_print
 from meta import VERSION, VERSION_BASE, VERSION_STRING, build, name as appname
+from multiprocess import mp
 from options import debug, verbose
 from util_str import safe_str, safe_unicode
 if sys.platform == "win32":
 	from util_win import win_ver
 	import ctypes
-from wxaddons import wx
-from wxwindows import BaseApp
+
 
 def _excepthook(etype, value, tb):
 	handle_error((etype, value, tb))
@@ -48,6 +48,7 @@ sys.excepthook = _excepthook
 
 
 def main(module=None):
+	mp.freeze_support()
 	if module:
 		name = "%s-%s" % (appbasename, module)
 	else:
@@ -70,6 +71,24 @@ def main(module=None):
 		# Linux
 		safe_print(' '.join(platform.dist()), platform.machine())
 	safe_print("Python " + sys.version)
+	# Enable faulthandler
+	try:
+		import faulthandler
+	except Exception, exception:
+		safe_print(exception)
+	else:
+		try:
+			faulthandler.enable(open(os.path.join(logdir, pyname +
+														  "-fault.log"), "w"))
+		except Exception, exception:
+			safe_print(exception)
+		else:
+			safe_print("Faulthandler", getattr(faulthandler, "__version__", ""))
+	from wxaddons import wx
+	if u"phoenix" in wx.PlatformInfo:
+		# py2exe helper so wx.xml gets picked up
+		from wx import xml
+	from wxwindows import BaseApp
 	safe_print("wxPython " + wx.version())
 	safe_print("Encoding: " + enc)
 	safe_print("File system encoding: " + fs_enc)
@@ -77,8 +96,8 @@ def main(module=None):
 		# HighDPI support
 		try:
 			shcore = ctypes.windll.shcore
-		except:
-			pass
+		except Exception, exception:
+			safe_print("Warning - could not load shcore:", exception)
 		else:
 			if hasattr(shcore, "SetProcessDpiAwareness"):
 				try:
@@ -88,6 +107,8 @@ def main(module=None):
 				except Exception, exception:
 					safe_print("Warning - SetProcessDpiAwareness() failed:",
 							   exception)
+			else:
+				safe_print("Warning - SetProcessDpiAwareness not found in shcore")
 	lockfilename = None
 	port = 0
 	# Allow multiple instances only for curve viewer, profile info,
@@ -290,6 +311,11 @@ def main(module=None):
 
 
 def _exit(lockfilename, port):
+	for process in mp.active_children():
+		if not "Manager" in process.name:
+			safe_print("Terminating zombie process", process.name)
+			process.terminate()
+			safe_print(process.name, "terminated")
 	for thread in threading.enumerate():
 		if (thread.isAlive() and thread is not threading.currentThread() and
 			not thread.isDaemon()):
