@@ -30,7 +30,7 @@ from wxwindows import (BaseApp, BaseFrame, BitmapBackgroundPanelText,
 					   CustomCheckBox, CustomGrid, CustomRowLabelRenderer,
 					   ConfirmDialog, FileDrop, InfoDialog, SimpleBook,
 					   TwoWaySplitter)
-from wxfixes import GenBitmapButton as BitmapButton
+from wxfixes import GenBitmapButton as BitmapButton, wx_Panel
 import colormath
 import config
 import wxenhancedplot as plot
@@ -284,9 +284,29 @@ class GamutCanvas(LUTCanvas):
 			min_x = -65.0
 			min_y = -65.0
 			step = 25
+		elif self.colorspace == "ICtCp":
+			label_x = "Ct"
+			label_y = "Cp"
+			max_x = 0.4
+			max_y = 0.5
+			min_x = -0.5
+			min_y = -0.4
+			step = .1
+		elif self.colorspace == "IPT":
+			label_x = "P"
+			label_y = "T"
+			max_x = 1.0
+			max_y = 1.0
+			min_x = -1.0
+			min_y = -1.0
+			step = .25
 		else:
-			label_x = "a*"
-			label_y = "b*"
+			if self.colorspace == "Lpt":
+				label_x = "p"
+				label_y = "t"
+			else:
+				label_x = "a*"
+				label_y = "b*"
 			max_x = 150.0
 			max_y = 150.0
 			min_x = -150.0
@@ -300,7 +320,11 @@ class GamutCanvas(LUTCanvas):
 						  "DIN99": lambda X, Y, Z: colormath.XYZ2DIN99(*[v * 100 for v in X, Y, Z])[1:],
 						  "DIN99b": lambda X, Y, Z: colormath.XYZ2DIN99b(*[v * 100 for v in X, Y, Z])[1:],
 						  "DIN99c": lambda X, Y, Z: colormath.XYZ2DIN99c(*[v * 100 for v in X, Y, Z])[1:],
-						  "DIN99d": lambda X, Y, Z: colormath.XYZ2DIN99d(*[v * 100 for v in X, Y, Z])[1:]}[self.colorspace]
+						  "DIN99d": lambda X, Y, Z: colormath.XYZ2DIN99d(*[v * 100 for v in X, Y, Z])[1:],
+						  "ICtCp": lambda X, Y, Z: colormath.XYZ2ICtCp(X, Y, Z,
+																	  clamp=False)[1:],
+						  "IPT": lambda X, Y, Z: colormath.XYZ2IPT(X, Y, Z)[1:],
+						  "Lpt": lambda X, Y, Z: colormath.XYZ2Lpt(*[v * 100 for v in X, Y, Z])[1:]}[self.colorspace]
 
 		if show_outline and (self.colorspace == "a*b*" or
 							 self.colorspace.startswith("DIN99")):
@@ -476,9 +500,9 @@ class GamutCanvas(LUTCanvas):
 				self.profiles[i] = None
 				continue
 
-			id = profile.calculateID(False)
 			check = self.profiles.get(i)
-			if (check and check.ID == id and intent == self.intent and
+			if (check and check.fileName == profile.fileName and
+				check.ID == profile.calculateID(False) and intent == self.intent and
 				direction == self.direction and order == self.order):
 				continue
 
@@ -632,17 +656,18 @@ class GamutCanvas(LUTCanvas):
 		self.order = order
 
 
-class GamutViewOptions(wx.Panel):
+class GamutViewOptions(wx_Panel):
 	
 	def __init__(self, *args, **kwargs):
 		scale = getcfg("app.dpi") / config.get_default_dpi()
 		if scale < 1:
 			scale = 1
 
-		wx.Panel.__init__(self, *args, **kwargs)
+		wx_Panel.__init__(self, *args, **kwargs)
 		self.SetBackgroundColour(BGCOLOUR)
 		self.sizer = wx.FlexGridSizer(0, 3, 4, 0)
 		self.sizer.AddGrowableCol(0)
+		self.sizer.AddGrowableCol(1)
 		self.sizer.AddGrowableCol(2)
 		self.SetSizer(self.sizer)
 
@@ -685,7 +710,8 @@ class GamutViewOptions(wx.Panel):
 		
 		self.sizer.Add((0, 0))
 		self.options_sizer = wx.FlexGridSizer(0, 3, 4, 8)
-		self.sizer.Add(self.options_sizer)
+		self.options_sizer.AddGrowableCol(2)
+		self.sizer.Add(self.options_sizer, flag=wx.EXPAND)
 		
 		# Colorspace select
 		self.colorspace_outline_bmp = wx.StaticBitmap(self, -1,
@@ -707,9 +733,12 @@ class GamutViewOptions(wx.Panel):
 														  "DIN99",
 														  "DIN99b",
 														  "DIN99c",
-														  "DIN99d"])
+														  "DIN99d",
+														  "ICtCp",
+														  "IPT",
+														  "Lpt"])
 		self.options_sizer.Add(self.colorspace_select, 
-							   flag=wx.ALIGN_CENTER_VERTICAL)
+							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.colorspace_select.Bind(wx.EVT_CHOICE, self.generic_select_handler)
 		self.colorspace_select.SetSelection(0)
 		
@@ -744,7 +773,7 @@ class GamutViewOptions(wx.Panel):
 														  lang.getstr("whitepoint.colortemp.locus.daylight"),
 														  lang.getstr("whitepoint.colortemp.locus.blackbody")])
 		self.options_sizer.Add(self.whitepoint_select, 
-							   flag=wx.ALIGN_CENTER_VERTICAL)
+							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.whitepoint_select.Bind(wx.EVT_CHOICE, self.generic_select_handler)
 		self.whitepoint_select.SetSelection(0)
 		self.whitepoint_bmp.Hide()
@@ -780,7 +809,7 @@ class GamutViewOptions(wx.Panel):
 												   choices=[])
 		self.comparison_profiles_sort()
 		self.options_sizer.Add(self.comparison_profile_select, 
-							   flag=wx.ALIGN_CENTER_VERTICAL)
+							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.comparison_profile_select.Bind(wx.EVT_CHOICE,
 											self.comparison_profile_select_handler)
 		droptarget = FileDrop(self.TopLevelParent,
@@ -806,7 +835,7 @@ class GamutViewOptions(wx.Panel):
 														  lang.getstr("gamap.intents.p"),
 														  lang.getstr("gamap.intents.s")])
 		self.options_sizer.Add(self.rendering_intent_select, 
-							   flag=wx.ALIGN_CENTER_VERTICAL)
+							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.rendering_intent_select.Bind(wx.EVT_CHOICE,
 										  self.rendering_intent_select_handler)
 		self.rendering_intent_select.SetSelection(0)
@@ -827,7 +856,7 @@ class GamutViewOptions(wx.Panel):
 										  choices=[lang.getstr("direction.forward"),
 												   lang.getstr("direction.backward.inverted")])
 		self.options_sizer.Add(self.direction_select,
-							   flag=wx.ALIGN_CENTER_VERTICAL)
+							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.direction_select.Bind(wx.EVT_CHOICE, self.direction_select_handler)
 		self.direction_select.SetSelection(0)
 		self.direction_select.Hide()
@@ -931,8 +960,11 @@ class GamutViewOptions(wx.Panel):
 				4: "DIN99",
 				5: "DIN99b",
 				6: "DIN99c",
-				7: "DIN99d"}.get(self.colorspace_select.GetSelection(),
-								 "a*b*" if dimensions == 2 else "Lab")
+				7: "DIN99d",
+				8: "ICtCp",
+				9: "IPT",
+				10: "Lpt"}.get(self.colorspace_select.GetSelection(),
+							   "a*b*" if dimensions == 2 else "Lab")
 
 	@property
 	def intent(self):
@@ -991,7 +1023,7 @@ class ProfileInfoFrame(LUTFrame):
 		self.splitter = TwoWaySplitter(self, -1, agwStyle = wx.SP_LIVE_UPDATE | wx.SP_NOSASH)
 		self.sizer.Add(self.splitter, 1, flag=wx.EXPAND)
 		
-		p1 = wx.Panel(self.splitter, name="canvaspanel")
+		p1 = wx_Panel(self.splitter, name="canvaspanel")
 		p1.SetBackgroundColour(BGCOLOUR)
 		p1.sizer = wx.BoxSizer(wx.VERTICAL)
 		p1.SetSizer(p1.sizer)
@@ -1079,7 +1111,7 @@ class ProfileInfoFrame(LUTFrame):
 		self.options_panel.AddPage(self.gamut_view_options, "")
 		
 		# Curve view options
-		self.lut_view_options = wx.Panel(p1, name="lut_view_options")
+		self.lut_view_options = wx_Panel(p1, name="lut_view_options")
 		self.lut_view_options.SetBackgroundColour(BGCOLOUR)
 		self.lut_view_options_sizer = self.box_sizer = wx.FlexGridSizer(0, 3, 4, 4)
 		self.lut_view_options_sizer.AddGrowableCol(0)
@@ -1163,7 +1195,7 @@ class ProfileInfoFrame(LUTFrame):
 		
 		self.lut_view_options_sizer.Add((0, 0))
 		
-		p2 = wx.Panel(self.splitter, name="gridpanel")
+		p2 = wx_Panel(self.splitter, name="gridpanel")
 		p2.sizer = wx.BoxSizer(wx.VERTICAL)
 		p2.SetSizer(p2.sizer)
 		self.splitter.AppendWindow(p2)
@@ -1425,8 +1457,10 @@ class ProfileInfoFrame(LUTFrame):
 												color.groups()[1].strip().split()],
 											  **dict(scale=255))
 				labelbgcolor = wx.Colour(*[int(round(v)) for v in color])
-				self.grid.SetRowLabelRenderer(i,
-											  CustomRowLabelRenderer(labelbgcolor))
+				rowlabelrenderer = CustomRowLabelRenderer(labelbgcolor)
+			else:
+				rowlabelrenderer = None
+			self.grid.SetRowLabelRenderer(i, rowlabelrenderer)
 			self.grid.SetCellTextColour(i, 0, textcolor)
 			self.grid.SetCellValue(i, 1, value)
 		
@@ -1495,6 +1529,15 @@ class ProfileInfoFrame(LUTFrame):
 						# DIN99
 						a, b = colormath.DIN992Lab(L99, xy[0], xy[1])[1:]
 					x, y = colormath.Lab2xyY(100.0, a, b)[:2]
+				elif page.colorspace == "ICtCp":
+					X, Y, Z = colormath.ICtCp2XYZ(1.0, xy[0], xy[1])
+					x, y = colormath.XYZ2xyY(X, Y, Z)[:2]
+				elif page.colorspace == "IPT":
+					X, Y, Z = colormath.IPT2XYZ(1.0, xy[0], xy[1])
+					x, y = colormath.XYZ2xyY(X, Y, Z)[:2]
+				elif page.colorspace == "Lpt":
+					X, Y, Z = colormath.Lpt2XYZ(100.0, xy[0], xy[1])
+					x, y = colormath.XYZ2xyY(X, Y, Z)[:2]
 				else:
 					# xy
 					x, y = xy
