@@ -391,6 +391,7 @@ class ReportFrame(BaseFrame):
 			else:
 				path = None
 			setcfg("measurement_report.output_profile", path)
+			XYZbpout = self.XYZbpout
 			# XYZbpout will be set to the blackpoint of the selected profile.
 			# This is used to determine if black output offset controls should
 			# be shown. Set a initial value slightly above zero so output
@@ -399,6 +400,7 @@ class ReportFrame(BaseFrame):
 		else:
 			profile = None
 			if which == "input":
+				XYZbpin = self.XYZbpin
 				self.XYZbpin = [0, 0, 0]
 		if path or profile:
 			if path and not os.path.isfile(path):
@@ -432,6 +434,7 @@ class ReportFrame(BaseFrame):
 					if (not getattr(self, which + "_profile", None) or
 						getattr(self, which + "_profile").fileName !=
 						profile.fileName):
+						# Profile selection has changed
 						if which == "simulation":
 							# Get profile blackpoint so we can check if it makes
 							# sense to show TRC type and output offset controls
@@ -440,10 +443,14 @@ class ReportFrame(BaseFrame):
 														   pcs="x")
 							except Exception, exception:
 								show_result_dialog(exception, self)
+								self.set_profile_ctrl_path(which)
+								return
 							else:
 								if len(odata) != 1 or len(odata[0]) != 3:
 									show_result_dialog("Blackpoint is invalid: %s"
 													   % odata, self)
+									self.set_profile_ctrl_path(which)
+									return
 								self.XYZbpin = odata[0]
 						elif which == "output":
 							# Get profile blackpoint so we can check if input
@@ -453,28 +460,42 @@ class ReportFrame(BaseFrame):
 														   pcs="x")
 							except Exception, exception:
 								show_result_dialog(exception, self)
+								self.set_profile_ctrl_path(which)
+								return
 							else:
 								if len(odata) != 1 or len(odata[0]) != 3:
 									show_result_dialog("Blackpoint is invalid: %s"
 													   % odata, self)
+									self.set_profile_ctrl_path(which)
+									return
 								self.XYZbpout = odata[0]
+					else:
+						# Profile selection has not changed
+						# Restore cached XYZbp values
+						if which == "output":
+							self.XYZbpout = XYZbpout
+						elif which == "input":
+							self.XYZbpin = XYZbpin
 					setattr(self, "%s_profile" % which, profile)
 					if not silent:
-						setcfg("measurement_report.%s_profile" % which, profile.fileName)
+						setcfg("measurement_report.%s_profile" % which,
+							   profile and profile.fileName)
 						if which == "simulation":
 							self.use_simulation_profile_ctrl_handler(None)
 						elif hasattr(self, "XYZbpin"):
 							self.mr_update_main_controls()
 					return profile
 			if path:
-				getattr(self,
-						"%s_profile_ctrl" %
-						which).SetPath(getcfg("measurement_report.%s_profile" % which))
+				self.set_profile_ctrl_path(which)
 		else:
 			setattr(self, "%s_profile" % which, None)
 			if not silent:
 				setcfg("measurement_report.%s_profile" % which, None)
 				self.mr_update_main_controls()
+
+	def set_profile_ctrl_path(self, which):
+		getattr(self, "%s_profile_ctrl" %
+					  which).SetPath(getcfg("measurement_report.%s_profile" % which))
 	
 	def mr_setup_language(self):
 		# Shared with main window
@@ -510,7 +531,8 @@ class ReportFrame(BaseFrame):
 		shown = self.apply_trc_ctrl.IsShown()
 		enable6 = (shown and
 				   bool(getcfg("measurement_report.apply_trc")))
-		show = shown and self.mr_trc_ctrl.GetSelection() == 2
+		show = shown and (self.mr_trc_ctrl.GetSelection() == 2 or
+						  getcfg("show_advanced_options"))
 		self.panel.Freeze()
 		self.mr_trc_ctrl.Enable(enable6)
 		self.mr_trc_ctrl.Show(shown)
@@ -569,7 +591,7 @@ class ReportFrame(BaseFrame):
 		self.output_profile_ctrl.SetPath(getcfg("measurement_report.output_profile"))
 		self.set_profile("output", silent=True)
 		self.mr_set_testchart(getcfg("measurement_report.chart"))
-		self.use_simulation_profile_ctrl_handler(None)
+		self.use_simulation_profile_ctrl_handler(None, update_trc=False)
 		self.panel.Thaw()
 
 	def mr_update_trc_control(self):
@@ -763,6 +785,13 @@ class ReportFrame(BaseFrame):
 				seconds += math.ceil(oseconds / 60.0) * tpp[1]
 				# Assume white patch every n samples
 				seconds += math.ceil(opatches / 40.0) * tpp[1]
+			if (which in ("testchart", "chart") and
+				getcfg("testchart.patch_sequence") !=
+				"optimize_display_response_delay"):
+				# Roughly 650s patch response delay per 1000 patches
+				# reduced by roughly 1 / 1.75 through optimization.
+				seconds -= 0.65 / 1.75 * patches
+				seconds += 0.65 * patches
 			timestamp = gmtime(seconds)
 			hours = int(strftime("%H", timestamp))
 			minutes = int(strftime("%M", timestamp))
@@ -794,7 +823,7 @@ class ReportFrame(BaseFrame):
 			   int(self.devlink_profile_cb.GetValue()))
 		self.mr_update_main_controls()
 	
-	def use_simulation_profile_ctrl_handler(self, event):
+	def use_simulation_profile_ctrl_handler(self, event, update_trc=True):
 		if event:
 			use_sim_profile = self.simulation_profile_cb.GetValue()
 			setcfg("measurement_report.use_simulation_profile",
@@ -812,27 +841,29 @@ class ReportFrame(BaseFrame):
 				sim_profile.tags.gTRC == sim_profile.tags.bTRC and
 				isinstance(sim_profile.tags.rTRC, ICCP.CurveType)):
 				tf = sim_profile.tags.rTRC.get_transfer_function()
-				# Use BT.1886 gamma mapping for SMPTE 240M / Rec. 709 TRC
-				setcfg("measurement_report.apply_trc",
-					   int(tf[0][1] in (-240, -709) or
-						   tf[0][0].startswith("Gamma")))
-				# Use only BT.1886 black output offset
-				setcfg("measurement_report.apply_black_offset",
-					   int(tf[0][1] not in (-240, -709) and
-						   not tf[0][0].startswith("Gamma") and
-						   self.XYZbpin != self.XYZbpout))
-				# Set gamma to profile gamma if single gamma profile
-				if tf[0][0].startswith("Gamma"):
-					if not getcfg("measurement_report.trc_gamma.backup", False):
-						# Backup current gamma
-						setcfg("measurement_report.trc_gamma.backup",
-							   getcfg("measurement_report.trc_gamma"))
-					setcfg("measurement_report.trc_gamma", round(tf[0][1], 2))
-				# Restore previous gamma if not single gamma profile
-				elif getcfg("measurement_report.trc_gamma.backup", False):
-					setcfg("measurement_report.trc_gamma",
-						   getcfg("measurement_report.trc_gamma.backup"))
-					setcfg("measurement_report.trc_gamma.backup", None)
+				if update_trc or self.XYZbpin == self.XYZbpout:
+					# Use only BT.1886 black output offset
+					setcfg("measurement_report.apply_black_offset",
+						   int(tf[0][1] not in (-240, -709) and
+							   (not tf[0][0].startswith("Gamma") or tf[1] < .95) and
+							   self.XYZbpin != self.XYZbpout))
+				if update_trc:
+					# Use BT.1886 gamma mapping for SMPTE 240M / Rec. 709 TRC
+					setcfg("measurement_report.apply_trc",
+						   int(tf[0][1] in (-240, -709) or
+							   (tf[0][0].startswith("Gamma") and tf[1] >= .95)))
+					# Set gamma to profile gamma if single gamma profile
+					if tf[0][0].startswith("Gamma") and tf[1] >= .95:
+						if not getcfg("measurement_report.trc_gamma.backup", False):
+							# Backup current gamma
+							setcfg("measurement_report.trc_gamma.backup",
+								   getcfg("measurement_report.trc_gamma"))
+						setcfg("measurement_report.trc_gamma", round(tf[0][1], 2))
+					# Restore previous gamma if not single gamma profile
+					elif getcfg("measurement_report.trc_gamma.backup", False):
+						setcfg("measurement_report.trc_gamma",
+							   getcfg("measurement_report.trc_gamma.backup"))
+						setcfg("measurement_report.trc_gamma.backup", None)
 				self.mr_update_trc_controls()
 				enable = (tf[0][1] not in (-240, -709) and
 						  self.XYZbpin != self.XYZbpout)
